@@ -2,9 +2,9 @@ from __future__ import unicode_literals
 import pytest
 import time
 
-import redis
-from redis.exceptions import ConnectionError
-from redis._compat import basestring, unichr
+import redis3
+from redis3.exceptions import ConnectionError
+from redis3._compat import basestring, unichr
 
 from .conftest import _get_client
 from .conftest import skip_if_server_version_lt
@@ -365,7 +365,7 @@ class TestPubSubAutoDecoding(object):
 
     @pytest.fixture()
     def r(self, request):
-        return _get_client(redis.Redis, request=request, decode_responses=True)
+        return _get_client(redis3.Redis, request=request, decode_responses=True)
 
     def test_channel_subscribe_unsubscribe(self, r):
         p = r.pubsub()
@@ -448,11 +448,20 @@ class TestPubSubAutoDecoding(object):
                                                  new_data,
                                                  pattern=self.pattern)
 
+    def test_context_manager(self, r):
+        with r.pubsub() as pubsub:
+            pubsub.subscribe('foo')
+            assert pubsub.connection is not None
+
+        assert pubsub.connection is None
+        assert pubsub.channels == {}
+        assert pubsub.patterns == {}
+
 
 class TestPubSubRedisDown(object):
 
     def test_channel_subscribe(self, r):
-        r = redis.Redis(host='localhost', port=6390)
+        r = redis3.Redis(host='localhost', port=6390)
         p = r.pubsub()
         with pytest.raises(ConnectionError):
             p.subscribe('foo')
@@ -531,3 +540,11 @@ class TestPubSubConnectionKilled(object):
                 r.client_kill_filter(_id=client['id'])
         with pytest.raises(ConnectionError):
             wait_for_message(p)
+
+
+class TestPubSubTimeouts(object):
+    def test_get_message_with_timeout_returns_none(self, r):
+        p = r.pubsub()
+        p.subscribe('foo')
+        assert wait_for_message(p) == make_message('subscribe', 'foo', 1)
+        assert p.get_message(timeout=0.01) is None
